@@ -22,6 +22,7 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <libhandy-1/handy.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -34,6 +35,7 @@
 #include "proctable.h"
 #include "procactions.h"
 #include "procdialogs.h"
+#include "setaffinity.h"
 #include "memmaps.h"
 #include "openfiles.h"
 #include "procproperties.h"
@@ -73,6 +75,19 @@ search_text_changed (GtkEditable *entry, gpointer data)
                                     GTK_TREE_VIEW (app->tree))))));
 }
 
+static void
+set_affinity_visiblity (GtkWidget *widget, gpointer user_data)
+{
+#ifndef __linux__
+    GtkMenuItem *item = GTK_MENU_ITEM (widget);
+    const gchar *name = gtk_menu_item_get_label (item);
+
+    if (strcmp (name, "Set _Affinity") == 0) {
+        gtk_widget_set_visible (widget, false);
+    }
+#endif
+}
+
 static void 
 create_proc_view(GsmApplication *app, GtkBuilder * builder)
 {
@@ -90,7 +105,9 @@ create_proc_view(GsmApplication *app, GtkBuilder * builder)
     GMenuModel *menu_model = G_MENU_MODEL (gtk_builder_get_object (builder, "process-popup-menu"));
     app->popup_menu = GTK_MENU (gtk_menu_new_from_model (menu_model));
     gtk_menu_attach_to_widget (app->popup_menu, GTK_WIDGET (app->main_window), NULL);
-    
+
+    gtk_container_foreach (GTK_CONTAINER (app->popup_menu), set_affinity_visiblity, NULL);
+
     app->search_bar = GTK_SEARCH_BAR (gtk_builder_get_object (builder, "proc_searchbar"));
     app->search_entry = GTK_SEARCH_ENTRY (gtk_builder_get_object (builder, "proc_searchentry"));
     
@@ -180,6 +197,7 @@ static void
 create_sys_view (GsmApplication *app, GtkBuilder * builder)
 {
     GtkBox *cpu_graph_box, *mem_graph_box, *net_graph_box;
+    GtkExpander *cpu_expander, *mem_expander, *net_expander;
     GtkLabel *label,*cpu_label;
     GtkGrid *table;
     GsmColorButton *color_picker;
@@ -201,6 +219,8 @@ create_sys_view (GsmApplication *app, GtkBuilder * builder)
     /* The CPU BOX */
     
     cpu_graph_box = GTK_BOX (gtk_builder_get_object (builder, "cpu_graph_box"));
+    cpu_expander = GTK_EXPANDER (gtk_builder_get_object (builder, "cpu_expander"));
+    g_object_bind_property (cpu_expander, "expanded", cpu_expander, "vexpand", G_BINDING_DEFAULT);
 
     cpu_graph = new LoadGraph(LOAD_GRAPH_CPU);
     gtk_widget_set_size_request (GTK_WIDGET(load_graph_get_widget(cpu_graph)), -1, 70);
@@ -242,7 +262,7 @@ create_sys_view (GsmApplication *app, GtkBuilder * builder)
         gtk_widget_show (GTK_WIDGET (label));
         g_free (label_text);
 
-        cpu_label = GTK_LABEL (gtk_label_new (NULL));
+        cpu_label = make_tnum_label ();
 
         /* Reserve some space to avoid the layout changing with the values. */
         gtk_label_set_width_chars(cpu_label, 6);
@@ -259,6 +279,8 @@ create_sys_view (GsmApplication *app, GtkBuilder * builder)
     /** The memory box */
     
     mem_graph_box = GTK_BOX (gtk_builder_get_object (builder, "mem_graph_box"));
+    mem_expander = GTK_EXPANDER (gtk_builder_get_object (builder, "mem_expander"));
+    g_object_bind_property (mem_expander, "expanded", mem_expander, "vexpand", G_BINDING_DEFAULT);
 
     mem_graph = new LoadGraph(LOAD_GRAPH_MEM);
     gtk_widget_set_size_request (GTK_WIDGET(load_graph_get_widget(mem_graph)), -1, 70);
@@ -299,6 +321,8 @@ create_sys_view (GsmApplication *app, GtkBuilder * builder)
     /* The net box */
     
     net_graph_box = GTK_BOX (gtk_builder_get_object (builder, "net_graph_box"));
+    net_expander = GTK_EXPANDER (gtk_builder_get_object (builder, "net_expander"));
+    g_object_bind_property (net_expander, "expanded", net_expander, "vexpand", G_BINDING_DEFAULT);
 
     net_graph = new LoadGraph(LOAD_GRAPH_NET);
     gtk_widget_set_size_request (GTK_WIDGET(load_graph_get_widget(net_graph)), -1, 70);
@@ -403,6 +427,13 @@ on_activate_about (GSimpleAction *, GVariant *, gpointer data)
         );
 }
 
+static void
+on_activate_keyboard_shortcuts (GSimpleAction *, GVariant *, gpointer data)
+{
+    GsmApplication *app = (GsmApplication *) data;
+    gtk_widget_show (GTK_WIDGET (gtk_application_window_get_help_overlay (GTK_APPLICATION_WINDOW (app->main_window))));
+}
+
 static void
 on_activate_refresh (GSimpleAction *, GVariant *, gpointer data)
 {
@@ -439,6 +470,14 @@ on_activate_send_signal (GSimpleAction *, GVariant *parameter, gpointer data)
     }
 }
 
+static void
+on_activate_set_affinity (GSimpleAction *, GVariant *, gpointer data)
+{
+    GsmApplication *app = (GsmApplication *) data;
+
+    create_set_affinity_dialog (app);
+}
+
 static void
 on_activate_memory_maps (GSimpleAction *, GVariant *, gpointer data)
 {
@@ -652,7 +691,7 @@ cb_main_window_state_changed (GtkWidget *window, GdkEventWindowState *event, gpo
 void
 create_main_window (GsmApplication *app)
 {
-    GtkApplicationWindow *main_window;
+    HdyApplicationWindow *main_window;
     GtkStack *stack;
     GMenuModel *window_menu_model;
     GMenuModel *process_menu_model;
@@ -666,22 +705,26 @@ create_main_window (GsmApplication *app)
     GtkBuilder *builder = gtk_builder_new();
     gtk_builder_add_from_resource (builder, "/org/gnome/gnome-system-monitor/data/interface.ui", NULL);
     gtk_builder_add_from_resource (builder, "/org/gnome/gnome-system-monitor/data/menus.ui", NULL);
+    gtk_builder_add_from_resource (builder, "/org/gnome/gnome-system-monitor/gtk/help-overlay.ui", NULL);
 
-    main_window = GTK_APPLICATION_WINDOW (gtk_builder_get_object (builder, "main_window"));
+    main_window = HDY_APPLICATION_WINDOW (gtk_builder_get_object (builder, "main_window"));
     gtk_window_set_application (GTK_WINDOW (main_window), app->gobj());
     gtk_widget_set_name (GTK_WIDGET (main_window), "gnome-system-monitor");
     app->main_window = main_window;
 
+    gtk_application_window_set_help_overlay (GTK_APPLICATION_WINDOW (app->main_window),
+                                             GTK_SHORTCUTS_WINDOW (gtk_builder_get_object (builder, "help_overlay")));
+
     session = g_getenv ("XDG_CURRENT_DESKTOP");
     if (session && !strstr (session, "GNOME")){
         GtkBox *mainbox;
-        GtkHeaderBar *headerbar;
+        HdyHeaderBar *headerbar;
 
         mainbox = GTK_BOX (gtk_builder_get_object (builder, "main_box"));
-        headerbar = GTK_HEADER_BAR (gtk_builder_get_object (builder, "header_bar"));
+        headerbar = HDY_HEADER_BAR (gtk_builder_get_object (builder, "header_bar"));
         gtk_style_context_remove_class (gtk_widget_get_style_context (GTK_WIDGET (headerbar)), "titlebar");
         gtk_window_set_titlebar (GTK_WINDOW (main_window), NULL);
-        gtk_header_bar_set_show_close_button (headerbar, FALSE);
+        hdy_header_bar_set_show_close_button (headerbar, FALSE);
         gtk_box_pack_start (mainbox, GTK_WIDGET (headerbar), FALSE, FALSE, 0);
     }
 
@@ -719,12 +762,14 @@ create_main_window (GsmApplication *app)
 
     GActionEntry win_action_entries[] = {
         { "about", on_activate_about, NULL, NULL, NULL },
+        { "show-help-overlay", on_activate_keyboard_shortcuts,  NULL, NULL, NULL},
         { "search", on_activate_search, "b", "false", NULL },
         { "send-signal-stop", on_activate_send_signal, "i", NULL, NULL },
         { "send-signal-cont", on_activate_send_signal, "i", NULL, NULL },
         { "send-signal-end", on_activate_send_signal, "i", NULL, NULL },
         { "send-signal-kill", on_activate_send_signal, "i", NULL, NULL },
         { "priority", on_activate_priority, "i", "@i 0", change_priority_state },
+        { "set-affinity", on_activate_set_affinity, NULL, NULL, NULL },
         { "memory-maps", on_activate_memory_maps, NULL, NULL, NULL },
         { "open-files", on_activate_open_files, NULL, NULL, NULL },
         { "process-properties", on_activate_process_properties, NULL, NULL, NULL },
@@ -809,6 +854,7 @@ update_sensitivity(GsmApplication *app)
                                               "send-signal-end",
                                               "send-signal-kill",
                                               "priority",
+                                              "set-affinity",
                                               "memory-maps",
                                               "open-files",
                                               "process-properties" };
